# -*- coding: utf-8 -*-
# raumplanerModell.py
## einfache Variante ohne Behandlung von Mausereignissen

"""Eine eigene Modellklasse für den Raumplaner
uebernimmt die gesamte Verwaltung der erzeugten Moebel
einschliesslich Speichern und Laden,
sowie Auswahl eines Objekts
Die Klasse ist als Singleton modelliert"""

import pickle # Persistenz realisieren

from grafikfenster import *

from badewanne import Badewanne
from bett import Bett
from dusche import Dusche
from herd import Herd
from klosett import Klosett
from sessel import Sessel
from schrank import Schrank
from sofa import Sofa
from stuhl import Stuhl
from tisch import Tisch
from ovalerTisch import OvalerTisch

### ---------------------------------------------------
class RaumplanerModell(object):
    '''Ist als Singleton modelliert'''

    __modell = None

    @staticmethod
    def GibModell():
        if RaumplanerModell.__modell == None:
            RaumplanerModell.__modell=RaumplanerModell()
        return RaumplanerModell.__modell
    
    def __init__(self):
        """Konstruktor"""
        ## Fuer Singleton: ----------------------------------------
        if RaumplanerModell.__modell != None:
            raise Exception('Konstruktor nicht direkt aufrufen')
        ## --------------------------------------------------------
        self.__alleMoebel = []
        self.__ausgewaehlt = -1

    #### ----- Erzeugen -----
    def Neu(self, klasse, *par, **kwargs):
        if klasse.__class__==str:
            klasse=eval(klasse)
        neu=klasse(*par, **kwargs)
        self.__alleMoebel.append(neu)
        return neu

    # Loeschen s.u.

    #### ----- sichern und laden -----
    def Speichere(self, dateiname):
        """Objektdaten werden in Datei geschrieben"""
        try:
            with open(dateiname, 'wb') as objekt_datei:
                pickle.dump(self.__alleMoebel, objekt_datei)
                return 'OK'
        except IOError as ioerr:
            return 'Dateifehler: ' + str(ioerr)
        except pickle.PicklingError as perr:
            return 'Pickle-Fehler: ' + str(perr)

    def Lade(self, dateiname):
        """Objektdaten werden aus der Datei gelesen"""
        for moebel in self.__alleMoebel:
            moebel.Verberge()
            del(moebel)
        self.__ausgewaehlt = -1
        try:
            with open(dateiname, 'rb') as objekt_datei:
                self.__alleMoebel = pickle.load(objekt_datei)
                for moebel in self.__alleMoebel:
                    if moebel.GibSichtbar():
                        moebel.Zeige()
                return 'OK'
        except IOError:
            return 'Datei nicht auffindbar!'
        except AttributeError as ae:
            return 'AttributeError: ' + str(ae)
        except pickle.UnpicklingError as perr:
            return 'Pickle-Fehler: ' + str(perr)

    def BildExport(self, dateiname):
        """Bild wird in Bild-Datei geschrieben"""
        Zeichenflaeche.GibZeichenflaeche().BildExport(dateiname)        
        
    #### ----- Auswahl -----
    def Waehle(self,index=None):
        '''wählt das Moebel aus;
        None zum Weitersetzen der Auswahl, -1 zum Beenden der Auswahl,
        nach dem letzten wird die Auswahl ebenfalls beendet'''
        if len(self.__alleMoebel)==0: # nichts auszuwaehlen
            return
        if index==None: # Weitersetzen
            if self.__ausgewaehlt!=-1:
                self.__alleMoebel[self.__ausgewaehlt].Waehle(False)
                self.__alleMoebel[self.__ausgewaehlt].Zeige()                
            self.__ausgewaehlt+=1
            if self.__ausgewaehlt==len(self.__alleMoebel):
                self.__ausgewaehlt=-1
            else:
                self.__alleMoebel[self.__ausgewaehlt].Waehle(True)
                self.__alleMoebel[self.__ausgewaehlt].Zeige()
            return
        # Indexfehler pruefen
        if index<-1: index=-1
        if index>=len(self.__alleMoebel): index=-1
        if self.__ausgewaehlt!=-1: # bisherige Auswahl beenden
            self.__alleMoebel[self.__ausgewaehlt].Waehle(False)
            self.__alleMoebel[self.__ausgewaehlt].Zeige()
        if index==-1: return # Auswahl ist geloescht
        self.__ausgewaehlt=index # Darstellung aktualisieren
        self.__alleMoebel[index].Waehle(True)

    def GibAusgewaehltes(self):
        if self.__ausgewaehlt<0: return None
        else: return self.__alleMoebel[self.__ausgewaehlt]

    def GibAktuelleDaten(self):
        if self.__ausgewaehlt<0: return None
        else:
            wahl=self.GibAusgewaehltes()
            wahl.Waehle(False)
            f=wahl.GibFarbe()
            wahl.Waehle(True)
            x,y=wahl.GibXPosition(),wahl.GibYPosition()
            w=wahl.GibOrientierung()
            return x,y,w,f

    def LoescheAusgewaehltes(self):
        if self.__ausgewaehlt>=0:
            self.GibAusgewaehltes().Verberge()
            self.__alleMoebel.remove(self.GibAusgewaehltes())
            self.__ausgewaehlt=-1

    def BearbeiteAusgewaehltes(self, **kwargs):
        """die kwargs dx,dy,dw,f erzeugen ein Dictionary"""
        if self.__ausgewaehlt>=0:
            gewaehlt=self.GibAusgewaehltes()
            if 'dx' in kwargs:
                gewaehlt.BewegeHorizontal(int(kwargs['dx']))
            if 'dy' in kwargs:
                gewaehlt.BewegeVertikal(int(kwargs['dy']))
            if 'dw' in kwargs:
                gewaehlt.Drehe(int(kwargs['dw']))
            if 'f' in kwargs:
                gewaehlt.AendereFarbe(kwargs['f'])
            if 'b' in kwargs:
                gewaehlt.AendereBreite(kwargs['b'])
            if 't' in kwargs:
                gewaehlt.AendereTiefe(kwargs['t'])

    def GibAlleMoebel(self):
        return self.__alleMoebel

### ---------------------------------------------------
class RaumplanerApp(wx.App):
    '''Hier nur fuer Testzwecke'''
    
    def OnInit(self):
        self.fenster = GrafikFenster(None, "Raumplaner-Grafik")
        self.SetTopWindow(self.fenster)
        self.fenster.Show(True)
        self.fenster.panel.Refresh()
        self.fenster.ZeigeShellFrame()
        #self.fenster.ZeigeFillingFrame()
        self.modell=RaumplanerModell.GibModell()
        self.TestAnwendung()
        return True

    def TestAnwendung(self):
        modell=self.modell
        modell.Neu(Stuhl,10, 30, 40, 40, 270, 'blue', True)
        modell.Neu(Tisch,60, 20, 120, 60, 0, 'red', True)
        modell.Neu(Schrank,280, 40, 100, 60, 90, 'brown', True)
        modell.Neu(Sessel,80, 100, 60, 60, 180, 'orange', True)

    def TestVeraenderndeMethoden(self):
        modell=self.modell
        modell.Waehle()
        modell.Waehle()
        modell.BearbeiteAusgewaehltes(dx=10,dy=20,dw=30,f='green',b=120,t=50)
        modell.Waehle(-1)

### ---------------------------------------------------
if __name__ == '__main__':
    app = RaumplanerApp(redirect=False)
    # Parameterwert True, wenn Ausgaben in der Standard E/A angezeigt werden sollen
    app.MainLoop()
